#include "nds.hh"
#include "nds_helper.hh"

#include <memory>
#include <cassert>
#include <cstdlib>
#include <iostream>
#include <sstream>
#include <string>

#include "nds_testing.hh"

std::ostream &
operator <<(std::ostream &os, const NDS::availability_list_type& avail)
{
  for (NDS::availability_list_type::const_iterator cur = avail.begin();
    cur != avail.end();
    ++cur)
  {
    os << cur->name << "(" << cur->data.size() << ")" << std::endl;
    for (NDS::segment_list_type::const_iterator curseg = cur->data.begin();
      curseg != cur->data.end();
      ++curseg)
    {
      os << "\t" << curseg->frame_type << " " << curseg->gps_start << "-" << curseg->gps_stop << std::endl;
    }
  }
  return os;
}

std::ostream &
operator <<(std::ostream &os, const NDS::simple_availability_list_type& avail)
{
  for (NDS::simple_availability_list_type::const_iterator cur = avail.begin();
    cur != avail.end();
    ++cur)
  {
    os << "(" << cur->size() << ")" << std::endl;
    for (NDS::simple_segment_list_type::const_iterator curseg = cur->begin();
      curseg != cur->end();
      ++curseg)
    {
      os << "\t" << curseg->gps_start << "-" << curseg->gps_stop << std::endl;
    }
  }
  return os;
}

int
main(int argc, char *argv[])
{
  using NDS::connection;

  //---------------------------------------------------------------------
  // Obtain the port of the server
  //---------------------------------------------------------------------
  connection::port_type	port = 31200;
  if ( ::getenv("NDS_TEST_PORT") ) 
  {
    std::istringstream	ps( ::getenv("NDS_TEST_PORT") );

    ps >> port;
  }

  std::string hostname("localhost");
  if ( ::getenv("NDS_TEST_HOST") ) 
  {
  	hostname = ::getenv( "NDS_TEST_HOST" );
  }

  connection::protocol_type proto = connection::PROTOCOL_TRY;
  if ( ::getenv("NDS_TEST_PROTO") )
  {
  	std::string ps( ::getenv("NDS_TEST_PROTO") );
  	if ( ps.compare("1") == 0 ) proto = connection::PROTOCOL_ONE;
  	else if ( ps.compare("2") == 0 ) proto = connection::PROTOCOL_TWO;
  }

  std::cerr << "Connecting to " << hostname << ":" << port << std::endl;
  pointer< connection > conn( new connection( hostname, port, proto ));

  NDS::buffer::gps_second_type gps_start = 1116733655;
  NDS::buffer::gps_second_type gps_stop = 1116733675;
  
  connection::channel_names_type cn;
  
  cn.push_back( "H1:SUS-ETMX_M0_MASTER_OUT_F1_DQ" );
  cn.push_back( "H1:SUS-ETMX_M0_MASTER_OUT_F2_DQ" );
  cn.push_back( "H1:SUS-ETMX_M0_MASTER_OUT_F3_DQ" );
  cn.push_back( "H1:SUS-ETMX_M0_MASTER_OUT_LF_DQ" );
  cn.push_back( "H1:SUS-ETMX_M0_MASTER_OUT_RT_DQ" );
  cn.push_back( "H1:SUS-ETMX_M0_MASTER_OUT_SD_DQ" );
  cn.push_back( "H1:SUS-ETMX_L1_MASTER_OUT_UL_DQ" );
  cn.push_back( "H1:SUS-ETMX_L1_MASTER_OUT_UR_DQ" );
  cn.push_back( "H1:SUS-ETMX_L1_MASTER_OUT_LL_DQ" );
  cn.push_back( "H1:SUS-ETMX_L1_MASTER_OUT_LR_DQ" );
  cn.push_back( "H1:SUS-ETMX_L2_MASTER_OUT_UL_DQ" );
  cn.push_back( "H1:SUS-ETMX_L2_MASTER_OUT_UR_DQ" );
  cn.push_back( "H1:SUS-ETMX_L2_MASTER_OUT_LL_DQ" );
  cn.push_back( "H1:SUS-ETMX_L2_MASTER_OUT_LR_DQ" );
  cn.push_back( "H1:SUS-ETMX_L3_MASTER_OUT_UL_DQ" );
  cn.push_back( "H1:SUS-ETMX_L3_MASTER_OUT_UR_DQ" );

  conn->set_epoch(gps_start, gps_stop);

  NDS::availability_list_type avail = conn->get_availability(cn);

  std::cerr << avail << std::endl;

  assert( cn.size() == avail.size() );
  for (int i = 0; i < cn.size(); ++i) 
  {
    assert ( cn[i] == avail[i].name );
    assert ( avail[i].data.size() == 2 );
    assert ( avail[i].data[0].frame_type == "H-H1_C" );
    assert ( avail[i].data[1].frame_type == "H-H1_R" );
    assert ( avail[i].data[0].gps_start == avail[i].data[1].gps_start);
    assert ( avail[i].data[0].gps_start == 1116733655 );
    assert ( avail[i].data[0].gps_stop == avail[i].data[1].gps_stop);
    assert ( avail[i].data[0].gps_stop    == 1116733675 );
  }

  std::cerr << std::endl << std::endl;

  {
    NDS::availability_list_type test_avail;
    NDS::availability entry;
    entry.name = std::string("X1:TEST_1");
    entry.data.push_back( NDS::segment( std::string("X-X1_C"),  1116733655, 1116733655+1024) );
    entry.data.push_back( NDS::segment( std::string("X-X1_R"),  1116733655, 1116733655+100) );
    entry.data.push_back( NDS::segment( std::string("X-X1_R"),  1116733655+1024, 1116733655+1024+100) );
    test_avail.push_back( entry );

    std::cerr << test_avail << std::endl;

    NDS::simple_availability_list_type simple_test = test_avail.simple_list();

    std::cerr << simple_test << std::endl;

    assert ( simple_test.size() == 1 );
    assert ( simple_test[0].size() == 1 );
    assert ( simple_test[0][0].gps_start == 1116733655 );
    assert ( simple_test[0][0].gps_stop == 1116733655+1024+100 );
  }
  // test the implicit availability call in fetch
  NDS::buffers_type bufs = conn->fetch(gps_start, gps_stop, cn);

  return 0;
}
